"""
Adapters for external systems and services.

These adapters implement the interfaces defined in solana_agent.interfaces
and provide concrete implementations for interacting with external systems.
"""
